/* ----------------------------------------------------------------------- *
 *   Copyright 1996-2004 Angel Marin - All Rights Reserved
 *   Angel Marin - anmar@gmx.net - http://anmar.eu.org/
 * 
 *   This file is part of rrdNetStats.
 *
 *   rrdNetStats is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   rrdNetStats is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with rrdNetStats; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ----------------------------------------------------------------------- */
#include "stdafx.h"
#include "cmdline.h"
#include <rrd_tool.h>

int CountArgs(char *aLine)
{
	int i=0;
	int aCount = 0;
	int inarg = 0;
	while (aLine[i] == ' ') i++;
	while (aLine[i] != 0) {
		if ((aLine[i]== ' ') && inarg) {
			inarg = 0;
		}
		if ((aLine[i]!= ' ') && ! inarg) {
			inarg = 1;
			aCount++;
		}
		i++;
	}
	return aCount;
}
/*
 * CreateArgs - take a string (aLine) and tokenize
 */
int CreateArgs(char *pName, char *aLine, int argc, char **argv)
{
	char *getP, *putP;
	char **pargv = argv;
	char Quote = 0;
	int inArg = 0;
	int len;

	len = strlen(aLine);
	/* remove trailing space and newlines */
	while (len && aLine[len] <= ' ') {
		aLine[len] = 0 ; len--;
	}
	/* sikp leading blanks */
	while (*aLine && *aLine <= ' ') aLine++;

	pargv[0] = pName;
	argc = 1;
	getP = aLine;
	putP = aLine;
	while (*getP){
		switch (*getP) {
			case ' ': 
				if (Quote){
					*(putP++)=*getP;
				} else if(inArg) {
					*(putP++) = 0;
					inArg = 0;
				}
				break;
			case '"':
			case '\'':
				if (Quote != 0) {
					if (Quote == *getP) {
						Quote = 0;
					} else {
						*(putP++)=*getP;
					}
				} else {
					if(!inArg) {
						pargv[argc++] = putP;
						inArg=1;
					}	    
					Quote = *getP;
				}
				break;
			default:
				if(!inArg) {
					pargv[argc++] = putP;
					inArg=1;
				}
				*(putP++)=*getP;
				break;
		}
		getP++;
	}
	*putP = '\0';
	return ((Quote)?-1:argc);
}

int create (char *database) {
	char *command, **myargv;
    int argc;

	if ( !database ) {
		fprintf (stderr, "Database file not specified !\n");
		goto end;
	}
	if (GetFileAttributes(database)!=-1) {
		fprintf (stderr, "Database file already exists !\n");
		goto end;
	}

	/* Prepare rrdtool comand line */
	command = (char *)malloc (sizeof (char) * (240 + strlen (database)));
	if ( !command ) {
		fprintf (stderr, "Error allocating memory !\n");
		goto end;
	}
	sprintf (command, "create %s --step 300 DS:input:COUNTER:600:0:U DS:output:COUNTER:600:0:U RRA:AVERAGE:0.5:1:800 RRA:AVERAGE:0.5:6:800 RRA:AVERAGE:0.5:24:800 RRA:AVERAGE:0.5:288:800 RRA:MAX:0.5:1:800 RRA:MAX:0.5:6:800 RRA:MAX:0.5:24:800 RRA:MAX:0.5:288:800", database);
    if ((argc = CountArgs(command)) == 0)  {
        fprintf (stderr,"ERROR: not enough arguments\n");		    
    }
    if ((myargv = (char **) malloc((argc+1) * sizeof(char *))) == NULL) {
        fprintf (stderr, "Error allocating memory !\n");
        goto end;
	}
    if ((argc=CreateArgs("", command, argc, myargv)) < 0) {
        fprintf (stderr, "ERROR: creating arguments\n");
        goto end;
    }
	/* Create the database */
    if (strcmp("create", myargv[1]) == 0) {
        optind=0; /* reset gnu getopt */
        opterr=0; /* no error messages */
        rrd_create(argc-1, &myargv[1]);
        if (rrd_test_error()) {
    	    printf("ERROR: %s\n",rrd_get_error());
	        rrd_clear_error();
        }
     }
end:
	if (command)
		free (command);
	if (myargv)
		free (myargv);

    return 0;
}

void get_interface_stats (const DWORD dwIndex, char *result) {
	MIB_IFROW *if_props;
	DWORD status, i = 0, dwSize;

	dwSize = sizeof(MIB_IFROW);
	if_props = (MIB_IFROW *)malloc(dwSize);
	if_props->dwIndex=dwIndex;
	status = GetIfEntry(if_props);
	if (status != ERROR_SUCCESS) {
		if (if_props)
			free (if_props);
		fprintf (stderr, "Error allocating memory !\n");
		sprintf(result, "N:0:0");
	} else {
		sprintf(result, "N:%u:%u", if_props->dwInOctets , if_props->dwOutOctets);
	}

	if (if_props)
		free (if_props);
	return;
}

int graph (const char *imagepath, char *database, const char *imagename, int type) {
	char *image, *definput, *defoutput, *command, *vrule1, *vrule2;
	char *imageparams [] = {"-108000", "day", "--x-grid HOUR:1:HOUR:6:HOUR:2:0:%H", "-691200", "week", "", "-3110400", "month", "", "-34214400", "year", ""};
    char **calcpr, **myargv;
    int argc, xsize, ysize;
    struct tm *stm;
	time_t v1_time, v2_time;

	image = (char *)malloc (strlen (imagepath) + strlen (imagename) + strlen (imageparams [type*3+1]) + 6 );

	if ( !image ) {
		fprintf (stderr, "Error allocating memory !\n");
		goto end;
	}
	if ( !database ) {
		database = (char *)malloc (strlen(imagename) + 5 );
		if ( !database ) {
			fprintf (stderr, "Error allocating memory !\n");
			goto end;
		}
		sprintf (database, "%s.rrd", imagename);
	}
	if (GetFileAttributes(database)==-1) {
		fprintf (stderr, "Database file not found !\n");
		goto end;
	}
  
	/* prepare VRULES */
	time(&v1_time);
	stm = localtime(&v1_time);
	stm->tm_hour = 0;
	stm->tm_sec = 0;
	stm->tm_min = 0;
	v1_time = mktime(stm);
	switch (type) {
		case 0:
			v2_time = v1_time - 86400;
			break;
		case 1:
			v1_time -= ((stm->tm_wday + 6) % 7 ) * 86400;
			v2_time = v1_time - (7*86400);
			break;
		case 2:
			stm->tm_mday = 1;
			v1_time = mktime(stm);
			stm->tm_mon--;
			v2_time = mktime(stm);
			break;
		case 3:
			stm->tm_mon = 0;
			stm->tm_mday = 1;
			v1_time = mktime(stm);
			stm->tm_year--;
			v2_time = mktime(stm);
			break;

	}
	vrule1 = (char *)malloc (strlen (database) + 25);
	vrule2 = (char *)malloc (strlen (database) + 25);
	if (!vrule1 || !vrule2 ) {
		fprintf (stderr, "Error allocating memory !\n");
		goto end;
	}
	sprintf (vrule1, "VRULE:%d#ff0000", v1_time);
	sprintf (vrule2, "VRULE:%d#ff0000", v2_time);

	/* Prepare defs*/
	definput = (char *)malloc (strlen (database) + 25); 
	defoutput = (char *)malloc (strlen (database) + 27);
	if ( !definput || !defoutput ) {
		fprintf (stderr, "Error allocating memory !\n");
		goto end;
	}
	sprintf (definput, "DEF:input=%s:input:AVERAGE", database);
	sprintf (defoutput, "DEF:output=%s:output:AVERAGE", database);

	/* Image file*/
	sprintf (image, "%s\\%s-%s.png", imagepath, imagename, imageparams [type*3+1]);

	/* Prepare rrdtool comand line */
	command = (char *)malloc (sizeof (char) * ( 700 + strlen (database) + strlen (imageparams [type*3]) + strlen (definput) + strlen (defoutput)));
	if ( !command ) {
		fprintf (stderr, "Error allocating memory !\n");
		goto end;
	}
	argc = sprintf (command, "graph %s --start %s --imgformat PNG --lazy  --vertical-label \"Bytes per second\" %s --alt-y-mrtg -c FONT#000000 -c MGRID#000000 -c FRAME#000000 -c BACK#f5f5f5 -c SHADEA#f5f5f5 -c SHADEB#f5f5f5 -c ARROW#ff0000 %s %s AREA:input#00cc00:\"Incoming traffic -\" GPRINT:input:MAX:\"Max\\: %%-9.2lf\" GPRINT:input:AVERAGE:\"Average\\: %%-9.2lf\" GPRINT:input:LAST:\"Current\\: %%-9.2lf\\l\" LINE1:output#0000FF:\"Outgoing traffic -\" GPRINT:output:MAX:\"Max\\: %%-9.2lf\" GPRINT:output:AVERAGE:\"Average\\: %%-9.2lf\" GPRINT:output:LAST:\"Current\\: %%-9.2lf\\l\" %s %s", image, imageparams [type*3], imageparams [type*3+2], definput, defoutput, vrule1, vrule2);
    if ((argc = CountArgs(command)) == 0)  {
        fprintf (stderr,"ERROR: not enough arguments\n");		    
    }
    if ((myargv = (char **) malloc((argc+1) * sizeof(char *))) == NULL) {
        fprintf (stderr, "Error allocating memory !\n");
        goto end;
	}
    if ((argc=CreateArgs("", command, argc, myargv)) < 0) {
        fprintf (stderr, "ERROR: creating arguments\n");
        goto end;
    }
	/* Create the graphic */
    if (strcmp("graph", myargv[1]) == 0) {
        optind=0; /* reset gnu getopt */
        opterr=0; /* no error messages */
        rrd_graph(argc-1, &myargv[1], &calcpr, &xsize, &ysize);
        if (rrd_test_error()) {
    	    printf("ERROR: %s\n",rrd_get_error());
	        rrd_clear_error();
        }
    }

end:
	if (image)
		free (image);
	if (definput)
		free (definput);
	if (defoutput)
		free (defoutput);
	if (command)
		free (command);
	if (vrule1)
		free (vrule1);
	if (myargv)
		free (myargv);

	return 0;
}

void list_interfaces () {
	MIB_IPADDRTABLE *if_list;
	MIB_IPADDRROW *if_addr_props;
	MIB_IFROW *if_props;
	DWORD status, i = 0, j, dwSize;

	dwSize = sizeof(MIB_IPADDRTABLE);
	if_list = (MIB_IPADDRTABLE *)malloc(dwSize);
	status = GetIpAddrTable(if_list, &dwSize, TRUE);
	if (status != ERROR_SUCCESS) {
		if (if_list)
			free (if_list);
		if_list = (MIB_IPADDRTABLE *)malloc(dwSize);
		status = GetIpAddrTable(if_list, &dwSize, TRUE);
	}
	dwSize = sizeof(MIB_IFROW);
	if_props = (MIB_IFROW *)malloc(dwSize);

	for (i = 0; i<if_list->dwNumEntries;i++) {
		if_addr_props = ( if_list->table ) +i;

		printf("\nInterface index: %u", if_addr_props->dwIndex);

		if_props->dwIndex=if_addr_props->dwIndex;
		status = GetIfEntry(if_props);
		if (status == ERROR_SUCCESS) {
			
			printf("\n\tName: %S", if_props->wszName);
			printf ("\n\tDescription: %s",if_props->bDescr);
			if (if_addr_props->dwAddr >0 ) {
				printf ("\n\tIP Address: %d.%d.%d.%d",(if_addr_props->dwAddr) & 0xFF, (if_addr_props->dwAddr >> 8) & 0xFF, (if_addr_props->dwAddr >> 16) & 0xFF, (if_addr_props->dwAddr >> 24) & 0xFF );
			}
			if ( if_props->dwPhysAddrLen >0 ) {
				printf("\n\tMAC Address: ");
				for (j = 0; j < if_props->dwPhysAddrLen; j++){
					printf(" %02x", if_props->bPhysAddr[j]);
				}
			}
			printf("\n");
		}
	}
	if (if_list)
		free (if_list);
	if (if_props)
		free (if_props);
	return;
}

int update (char *database, const unsigned long interface) {
	char *traffic = NULL, *command, **myargv;
    int argc;

	if (!interface) {
		fprintf (stderr, "Interface not specified !\n");
		goto end;
	}
	if ( !database ) {
		database = (char *)malloc (sizeof (char) * 10);
		if ( !database ) {
			fprintf (stderr, "Error allocating memory !\n");
			goto end;
		}
		sprintf (database, ".\\eth%u.rrd",interface);
	}
	
	if (GetFileAttributes(database)==-1) {
		fprintf (stderr, "Database file not found !\n");
		goto end;
	}

	/* Get traffic data */
	traffic = (char *)malloc (sizeof(char)*30);
	if ( !traffic ) {
		fprintf (stderr, "Error allocating memory !\n");
		goto end;
	}
	get_interface_stats (interface, traffic);

	if (strlen(traffic)>4) {
		/* Prepare rrdtool comand line */
		command = (char *)malloc (sizeof (char) * ( 10 + strlen (database) +strlen (traffic) ));
		if ( !command ) {
			fprintf (stderr, "Error allocating memory !\n");
			goto end;
		}
        sprintf (command, "update %s %s", database, traffic);
        if ((argc = CountArgs(command)) == 0)  {
            fprintf (stderr,"ERROR: not enough arguments\n");		    
        }
        if ((myargv = (char **) malloc((argc+1) * sizeof(char *))) == NULL) {
            fprintf (stderr, "Error allocating memory !\n");
            goto end;
	    }
        if ((argc=CreateArgs("", command, argc, myargv)) < 0) {
            fprintf (stderr, "ERROR: creating arguments\n");
            goto end;
        }
		/* Update the database */
        if (strcmp("update", myargv[1]) == 0) {
            optind=0; /* reset gnu getopt */
            opterr=0; /* no error messages */
            rrd_update(argc-1, &myargv[1]);
            if (rrd_test_error()) {
    	        printf("ERROR: %s\n",rrd_get_error());
	            rrd_clear_error();
            }
        }
	}

end:
	if (traffic)
		free (traffic);
	if (database)
		free (database);
	if (command)
		free (command);
	if (myargv)
		free (myargv);
	return 0;
}

int _tmain(int argc, _TCHAR* argv[])
{
	gengetopt_args_info args_info;
	if (cmdline_parser (argc, argv, &args_info) == 0) {
		if ( args_info.background_given ) {
			FreeConsole();
		}
		if ( args_info.list_given ) {
			list_interfaces();
		} else if ( args_info.graph_given ) {
			for ( int action=0;action<4;action++ ) {
				graph (args_info.path_img_arg, args_info.database_arg, args_info.graph_arg, action);
			}
		} else if ( args_info.update_given ) {
			update ( args_info.database_arg, args_info.update_arg );
		} else if ( args_info.create_given ) {
			create ( args_info.create_arg );
		}
	}
	return 0;
}
